__author__ = "iiPython"
__version__ = "1.1.6-3"
__license__ = "MIT"
__copyright__ = "Copyright 2022 iiPython"

from .iikp import readchar, keys
from .term import color, clear, cprint, to_ansi, colormap
